# -*- coding: utf-8 -*-

import giraffe
from giraffe.utils import HTMLWriter, camelcase2lower

ANCHOR=u"⚓"

def write_namespace (ns, stream):
	w = HTMLWriter (stream)
	classes = ns.classes + ns.interfaces
	classes.sort (lambda x, y: cmp(x.c_type, y.c_type))
	for cl in classes:
		write_class (cl, w)

def type_list (nodes, klass=""):
	"""Build a list of strings - possibly with markup - for a list of giraffe.ast.TypedNodes or Types"""
	result = []
	for n in nodes:
		if isinstance (n, giraffe.ast.TypedNode):
			t = n.type
		elif isinstance (n, giraffe.ast.Type):
			t = n
		else:
			raise TypeError, "Can not build type list with member of type %s: %s" % (n.__class__, n)
		
		if t.namespaced_name in t.namespace.repository:
			result.append(t.html_link(type_label (t), klass="type %s" % klass))
		else:
			result.append('<span class="type %s">%s</span>' % (klass, type_label (t)))
	return result

def param_list (nodes, klass=""):
	"""Build a list of method- or signal paramters - for a list of giraffe.ast.Parameters"""
	result = []
	for p in nodes:
		if not isinstance (p, giraffe.ast.Parameter):
			raise TypeError, "Can not build param list with member of type %s: %s" % (p.__class__, p)
		
		p_name = p.name
		if p.is_array:
			p_name = "**" + p_name
		elif p.is_out or p.is_pointer:
			p_name = "*" + p_name
		
		if p.type.namespaced_name in p.namespace.repository:
			p_type = p.type.html_link(type_label (p.type), klass="type %s" % klass)
		else:
			p_type = '<span class="type %s">%s</span>' % (klass, type_label (p.type))
		
		result += ["%s %s" % (p_type, p_name)]
	return result

def type_label (t):
	try:
		return t.c_identifier
	except:
		return t.c_type

def write_class (cl, w):
	#
	# Class header
	#
	if isinstance (cl, giraffe.ast.Interface):
		w.start_div (klass="class-interface")
		w.a(ANCHOR, href="#"+cl.anchor, klass="class-anchor")
		w.h2 ("Interface: %s" % cl.c_type, klass="class-interface", id=cl.anchor)
	elif cl.is_abstract:
		w.start_div (klass="class-abstract")
		w.a(ANCHOR, href="#"+cl.anchor, klass="class-anchor")
		w.h2("Abstract Class: %s" % cl.c_type, klass="class-abstract", id=cl.anchor)
	else:
		w.start_div (klass="class-normal")
		w.a(ANCHOR, href="#"+cl.anchor, klass="class-anchor")
		w.h2("Class: %s" % cl.c_type, klass="class-normal", id=cl.anchor)
	
	#
	# Class relations and interfaces
	#
	w.start_div (klass="relations")
	if isinstance (cl, giraffe.ast.Interface) and cl.implementations:
			w.start_p ()
			w.raw ('Classes implementing <span class="type">%s</span>: %s' % (cl.c_type, ", ".join (type_list(cl.implementations))))
			w.end_p ()
	if cl.interfaces:
		w.start_p ()
		w.raw ("Implements: %s" % (", ".join (type_list (cl.interfaces))))
		w.end_p ()
	if cl.subclasses:
		w.start_p ()
		w.raw ("Known subclasses: %s" % (", ".join (type_list (cl.subclasses))))
		w.end_p ()
	if cl.parent_class:
		w.start_p ()
		w.raw ("Subclass of: %s" % type_list ([cl.parent_class])[0])
		w.end_p ()
	if cl.doc : w.p(cl.doc)
	w.end_div()
	
	#
	# Properties
	#
	if cl.properties :
		w.start_div (klass="properties")
		w.h4("Properties", klass="properties")		
	for prop in cl.properties:
		access = " read-only"
		if prop.construct_only :
			access = "construct-only"
		elif prop.writable:
			access = "read/write"
		w.start_div (klass="property")
		w.start_p()
		w.span(prop.type.c_type, klass="property-type")
		w.span('"%s"' % prop.name, klass="property-name")
		w.span (access, klass="property-access")
		w.end_p()
		if prop.doc:
			w.p (prop.doc, klass="property-doc")
		w.end_div()
	if cl.properties:
		w.end_div()
	
	#
	# Signals
	#
	if cl.signals :
		w.start_div (klass="signals")
		w.h4("Signals", klass="signals")		
	for sig in cl.signals:
		params = ["%s %s" % (p.type.c_type, p.name) for p in sig.parameters]
		params = ["%s *%s" % (cl.c_type, camelcase2lower(cl.name))] + params
		
		if sig.return_value.type.name == "none":
			retval = "void"
		else:
			retval = sig.return_value.type.c_type
		w.start_div (klass="signal")
		w.start_p ()
		w.span(retval, klass="signal-return-type")
		w.span('"%s"' % sig.name, klass="signal-name")
		w.span ("(%s)" % ", ".join(params), klass="signal-parameters")
		w.end_p()
		if sig.doc:
			w.p (sig.doc, klass="signal-doc")
		w.end_div()
	if cl.signals:
		w.end_div()
	
	#
	# Methods
	#	
	if cl.methods :
		w.start_div(klass="methods")
		w.h4 ("Methods")		
	methods = cl.methods
	methods.sort(lambda x,y: cmp(x.c_identifier, y.c_identifier))
	for m in methods:					

		params = param_list (m.parameters)
		if not m.static:
			params = ["%s *self" % cl.c_type] + params
		
		if m.return_value.type.name == "none":
			retval = "void"
		elif m.return_value.is_array:
			retval = m.return_value.type.c_type + "*"
		else:
			retval = m.return_value.type.c_type
		
		cssklass = "method"
		if m.virtual : cssklass += " method-virtual"
		if m.static : cssklass += " method-static"
		w.start_div(klass=cssklass)
		w.start_p ()
		w.span(retval, klass="method-return-type")
		w.span(m.c_identifier, klass="method-name")
		w.start_span(klass="method-parameters")
		w.raw ("(" + ", ".join (params) + ")")
		w.end_span ()
		w.end_p()
		w.end_div()
	if cl.methods:
		w.end_div()
	
	w.end_div() # ends class div


